/*
** Module   : CommonFileInfo.java
** Abstract : Define all APIs available via the FILE-INFO system handle.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 CA  20130112 Created initial version.
*/

package com.goldencode.p2j.util;

/**
 * Exposes the following methods or attributes from the FILE-INFO handle:
 * <table>
 * <tr>
 *    <th>FILE-INFO attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>FILE-CREATE-DATE</td>
 *    <td>{@link #fileInfoGetCreationDate()}</td>
 * </tr>
 * <tr>
 *    <td>FILE-CREATE-TIME</td>
 *    <td>{@link #fileInfoGetCreationTime()}</td>
 * </tr>
 * <tr>
 *    <td>FILE-MOD-DATE</td>
 *    <td>{@link #fileInfoGetModDate()}</td>
 * </tr>
 * <tr>
 *    <td>FILE-MOD-TIME</td>
 *    <td>{@link #fileInfoGetModTime()}</td>
 * </tr>
 * <tr>
 *    <td>FILE-NAME</td>
 *    <td>{@link #initFileInfo(character)}, {@link #initFileInfo(String)} and
 *        {@link #getFileName()}
 *    </td>
 * </tr>
 * <tr>
 *    <td>FILE-SIZE</td>
 *    <td>{@link #fileInfoGetSize()}</td>
 * </tr>
 * <tr>
 *    <td>FILE-TYPE</td>
 *    <td>{@link #fileInfoGetType()}</td>
 * </tr>
 * <tr>
 *    <td>FULL-PATHNAME</td>
 *    <td>{@link #fileInfoGetFullPath()}</td>
 * </tr>
 * <tr>
 *    <td>PATHNAME</td>
 *    <td>{@link #fileInfoGetPathName()}</td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrapFileInfo(handle)} API, which, depending on the 
 * handle's state, it either returns an {@link FileSystemOps#asHandle()} 
 * instance or a dynamic proxy using {@link handle#invalidAttrAccessProxy}. 
 */
public interface CommonFileInfo
extends CommonFile,
        CommonHandle
{
   /**
    * Search for the specified file and initialize the related data
    * structures if found.  Relative filenames are searched for via the 
    * PROPATH.  If <code>name</code> is <code>unknown</code> or not found,
    * the current data structures are cleared.
    *
    * @param    name
    *           Relative or absolute filename to obtain information on.
    */
   public void initFileInfo(character name);
   
   /**
    * Search for the specified file and initialize the related data
    * structures if found.  Relative filenames are searched for via the 
    * {@link EnvironmentOps#getSearchPath}.  If <code>name</code> is
    * <code>unknown</code> or not found, the current data structures are
    * cleared.
    *
    * @param    name
    *           Relative or absolute filename to obtain information on.
    */
   public void initFileInfo(String name);
   
   /**
    * If the current file information data structures are initialized, return
    * the absolute or relative path name, otherwise return the
    * <code>unknown value</code>. If the specified file name was absolute,
    * then an absolute pathname will be returned.  Otherwise the relative
    * path that was found via {@link FileSystemOps#searchPath} will be 
    * returned.
    *
    * @return   The path name of the current file being inspected or the
    *           <code>unknown value</code> if the data structures were never
    *           initialized or if initialization failed.
    */
   public character fileInfoGetPathName();
   
   /**
    * If the current file information data structures are initialized, return
    * the absolute (canonicalized) path name, otherwise return the
    * <code>unknown value</code>.
    *
    * @return   The absolute path name of the current file being inspected or
    *           the <code>unknown value</code> if the data structures were
    *           never initialized or if initialization failed.
    */
   public character fileInfoGetFullPath();
   
   /**
    * If the current file information data structures are initialized, return
    * the file's size, otherwise return the <code>unknown value</code>.
    *
    * @return   The size in bytes of the current file being inspected or the
    *           <code>unknown value</code> if the data structures were never
    *           initialized or if initialization failed.
    */
   public integer fileInfoGetSize();
   
   /**
    * If the current file information data structures are initialized, return
    * a string with characters that specify attributes about the file,
    * otherwise return the <code>unknown value</code>.
    * <p>
    * One of the following mutually exclusive characters will be present:
    * <p>
    * <pre>
    * Character   Meaning
    * ---------   -----------------------------
    * D           directory
    * F           file
    * X           unknown
    * </pre>
    * <p>
    * <b>There is no support for "S" (UNIX special devices) or "M" (procedure
    * library members) at this time.</b>
    * <p>
    * One or more of the following characters will be present:
    * <p>
    * <pre>
    * Character   Meaning
    * ---------   -----------------------------
    * H           hidden
    * R           readable
    * W           writable
    * </pre>
    * <p>
    * <b>There is no support for "L" (UNIX symbolic links) or "P" (pipes)
    * at this time.</b>
    *
    * @return   The file attributes of the current file being inspected or
    *           the <code>unknown value</code> if the data structures were
    *           never initialized or if initialization failed.
    */
   public character fileInfoGetType();
   
   /**
    * If the current file information data structures are initialized, return
    * the file's last modification date, otherwise return the 
    * <code>unknown value</code>.
    *
    * @return   The last modification date of the current file being inspected
    *           or the <code>unknown value</code> if the data structures were
    *           never initialized or if initialization failed.
    */
   public date fileInfoGetModDate();
   
   /**
    * If the current file information data structures are initialized, return
    * the file's last modification time (in seconds from midnight), otherwise 
    * return the <code>unknown value</code>.
    *
    * @return   The last modification time in seconds from midnight of the
    *           current file being inspected or the <code>unknown value</code>
    *           if the data structures were never initialized or if
    *           initialization failed.
    */
   public integer fileInfoGetModTime();
   
   /**
    * If the current file information data structures are initialized, return
    * the file's creation date, otherwise return the 
    * <code>unknown value</code>.
    *
    * @return   Always <code>unknown value</code> at this time.
    */
   public date fileInfoGetCreationDate();
   
   /**
    * If the current file information data structures are initialized, return
    * the file's creation time (in seconds from midnight), otherwise 
    * return the <code>unknown value</code>.
    *
    * @return   Always returns <code>unknown value</code> at this time.
    */
   public integer fileInfoGetCreationTime();
}
