/*
** Module   : memptr.java
** Abstract : Progress 4GL compatible memptr data type
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ------------------Description-------------------
** 001 GES 20050906   @22564 Created initial version with support for a
**                           subset of memptr processing.
** 002 GES 20060422   @25654 Added a copy constructor.
** 003 GES 20060724   @28172 Use forced assign in copy constructor.
** 004 GES 20060807   @28476 Moved to externalizable interface to optimize
**                           network performance.
** 005 CA  20130123          Added c'tor which builds a new instance from a
**                           BaseDataType value, if possible. This should be
**                           used only when constructing an instance using the
**                           value returned by a DYNAMIC-FUNCTION call; 4GL
**                           does some special casting in this case, which is
**                           not implemented yet.
*/

package com.goldencode.p2j.util;

import java.io.*;

/**
 * A class that implements common services for a Progress 4GL compatible
 * memptr data type whose data is mutable.  Only a subset of the Progress
 * language features are supported.
 * <p>
 * In addition to the implementation of many built-in functions
 * and operators, this class hides the representation of the 
 * <code>unknown value</code> and properly maintains the processing of
 * <code>unknown value</code>. 
 * <p>
 * Format string related processing is handled in this class, see method
 * {@link #toString(String)}.
 * <p>
 * The following is the mapping of Progress language features to the
 * corresponding feature in this class:
 * <p>
 * <pre>
 * Progress Function               Java Method
 * -----------------------------   -----------------------
 * get-size                        {@link #length}
 *
 * Progress Language Statement     Java Method
 * -----------------------------   -----------------------
 * set-size                        {@link #setLength}
 * </pre> 
 * <p>
 * All other <code>MEMPTR</code>features are implemented in the common
 * base class <code>BinaryData</code> which is shared with <code>RAW</code>
 * types.
 *
 * @author    GES
 */
public class memptr
extends BinaryData
{
   /**
    * Stores the size that was last set so that a <code>null</code> value
    * can still report its size.
    */
   private int size = 0;
   
   /**
    * Default constructor, creates an instance that represents an
    * uninitialized byte array (a <code>null</code> array).
    */
   public memptr()
   {
      value = null;
   }

   /**
    * This is a special c'tor which should be used only when converting the
    * value returned by a DYNAMIC-FUNCTION call to the expected type (i.e
    * when passing that value as a parameter to another function or procedure
    * call).  In such cases, 4GL does some automatic conversion, which is not
    * implemented yet.
    * <p>
    * This version of the c'tor always sets the value to unknown.
    * 
    * @param    value
    *           The value to be used for this instance.
    */
   public memptr(BaseDataType value)
   {
      this();
      // no-op
   }
   
   /**
    * Constructs an instance after copying the parameter's data into the
    * internal representation of this class.
    * <p>
    * If the parameter's represents the <code>unknown value</code>, this
    * instance will also represent the <code>unknown value</code>.
    *
    * @param   value
    *          The value to be used for this instance.
    */
   public memptr(memptr value)
   {
      assign(value, true);
      size = value.size;
   }
   
   /**
    * Does the same as standard <code>clone()</code> method but returns an
    * instance of <code>BaseDataType</code> and doesn't throw the
    * <code>CloneNotSupportedException</code>.
    *
    * @return   A clone of this instance.
    */
   public BaseDataType duplicate()
   {
      memptr m = new memptr();
      
      // set the instance data
      m.assign(this, true);
      m.size = this.size;
      
      return m;
   }

   /**
    * Creates a new instance of the same type that represents the 
    * <code>unknown value</code>.
    *
    * @return   An instance that represents the <code>unknown value</code>.
    */
   public BaseDataType instantiateUnknown()   
   {
      memptr m = new memptr();
      m.setUnknown();
      
      return m;
   }
   
   /**
    * Returns the length this instance's data in bytes.
    *
    * @return   The number of bytes in this instance or <code>unknown</code>.
    */
   public integer length()
   {
      if (value == null && !unknown)
         return new integer(size);
      
      // this will handle the unknown value condition too
      return super.length();
   }
   
   /**
    * Returns the length this instance's data in bytes.
    *
    * @return   The number of bytes in this instance or -1 if this instance
    *           is <code>unknown</code>.
    */
   public int lengthOf()
   {
      if (value == null && !unknown)
         return size;
      
      // this will handle the unknown value condition too
      return super.lengthOf();
   }
   
   /**
    * Sets the array to a given length.  If set to 0, this de-allocates any
    * currently allocated array (if no array is allocated, this does
    * nothing).  If set to a positive number, this will allocate a new 
    * array if no array is allocated, otherwise this method will do nothing.
    *
    * @param    len
    *           New length of the array.
    *
    * @return   The current instance.
    */
   public BinaryData setLength(int len)
   {
      // are we already initialized?
      if (value == null)
      {
         if (len > 0)
         {
            // "allocate" our memory
            value = new byte[len];
         }
         else
         {
            // remember our size because Progress reports this even when no
            // memory is allocated, otherwise we ignore this
            size = len;
         }
      }
      else
      {
         // ignore anything but a de-allocation request
         if (len == 0)
         {
            // de-allocate our memory
            value = null;
         }
      }
      
      return this;
   }
   
   /**
    * Determines if the array of binary data stored in an instance of this
    * class must be limited in length.
    *
    * @return    <code>true</code> if the length must be limited.
    */
   public boolean isLengthLimited()
   {
      return false;
   }
   
   /**
    * Determines the maximum length of the array of binary data stored in an
    * instance of this class.
    *
    * @return    The length limit for binary data or -1 if this class has
    *            no limit.
    */
   public int lengthLimit()
   {
      return 0;
   }
   
   /**
    * Determines if the array of binary data stored in an instance of this
    * class must be automatically extended in length when operations
    * write past the current length.  Such extensions will pad with
    * <code>null</code> bytes.
    *
    * @return    <code>true</code> if the length must automatically extended.
    */
   public boolean isAutoExtend()
   {
      return false;
   }
   
   /**
    * Creates a string representation of the instance data, since a memptr
    * represents a 'pointer' it doesn't have a real string representation.
    * If the instance represents the <code>unknown value</code>, a '?' will
    * be returned, otherwise the empty string will be returned.
    *
    * @return   The empty string or '?'.
    */
   public String toString()   
   {
      return toString(null);
   }
   
   /**
    * Creates a string representation of the instance data using the given
    * format string, since a memptr represents a 'pointer' it doesn't have
    * a real string representation and the format string is meaningless.
    * If the instance represents the <code>unknown value</code>, a '?' will
    * be returned, otherwise the empty string will be returned.
    *
    * @param    fmt
    *           The format string to use.
    *
    * @return   The empty string or '?'.
    */
   public String toString(String fmt)   
   {
      if (unknown)
      {
         return "?";
      }
      
      return "";
   }
   
   /**
    * Creates a string representation of the instance data in a form that is
    * compatible with the <code>MESSAGE</code> language statement.  If the
    * instance represents the <code>unknown value</code>, a '?' will be
    * returned.
    * <p>
    * This is equivalent to {@link #toString}.
    *
    * @return   The 'message' formatted string.
    */
   public String toStringMessage()   
   {
      return toString();
   }
   
   /**
    * Creates a string representation of the instance data using the 'export'
    * format.  If the instance represents the <code>unknown value</code>, a
    * '?' will be returned.  Any other data is directly output into the
    * string and no formatting or conversion of binary data is done.
    *
    * @return   The binary data in the instance as a string.
    */
   public String toStringExport()   
   {
      if (unknown)
      {
         return "?";
      }
      
      return new String(value);
   }   
      
   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      size = in.readInt();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeInt(size);
   }   
}
